// File: SpellChecker.h
// Description: Declaration of external interfaces for SpellChecker module.
// Copyright: (C) 2005 EffectiveSoft Ltd. All Rights Reserved.
// Technical support: technical-support@effective-soft.com

#ifndef SEM_SDK_SPELLCHECKER__H_
#define SEM_SDK_SPELLCHECKER__H_

#include "BaseInterface.h"

#ifdef WIN32
// default : SEM_SDK_SPELLCHECKER_DLL_USE
#	if !defined(SEM_SDK_SPELLCHECKER_LIB_USE) \
			&& !defined(SEM_SDK_SPELLCHECKER_DLL_USE)	 \
			&& !defined(SEM_SDK_SPELLCHECKER_CREATE_DLL) 
#		define SEM_SDK_SPELLCHECKER_DLL_USE
#	endif

#	if defined(SEM_SDK_SPELLCHECKER_LIB_USE)
#		define SEM_SDK_SPELLCHECKER_
#		ifndef SEM_SDK_SPELLCHECKER_NO_AUTOLINK
#			if defined(_DEBUG) && !defined(SEM_SDK_NO_DEBUG_LIBS)
#				pragma comment(lib, "SpellChecker_SD.lib")
#			else
#				pragma comment(lib, "SpellChecker_S.lib")
#			endif //	_DEBUG
#		endif	  //	SEM_SDK_SPELLCHECKER_NO_AUTOLINK
#	elif defined(SEM_SDK_SPELLCHECKER_DLL_USE)
#		define SEM_SDK_SPELLCHECKER_ __declspec(dllimport)
#			ifndef SEM_SDK_SPELLCHECKER_NO_AUTOLINK
#				if defined(_DEBUG) && !defined(SEM_SDK_NO_DEBUG_LIBS)
#					pragma comment(lib, "SpellChecker_D.lib")
#				else
#					pragma comment(lib, "SpellChecker.lib")
#				endif // _DEBUG
#			endif // SEM_SDK_SPELLCHECKER_NO_AUTOLINK
#	elif defined(SEM_SDK_SPELLCHECKER_CREATE_DLL)
#		define SEM_SDK_SPELLCHECKER_ __declspec(dllexport)
#	endif
#else // WIN32
#define SEM_SDK_SPELLCHECKER_
#endif

namespace NsSemSDK
{
/// type of candidate weight
typedef unsigned char Weight_t;

/// Provide access to set of found candidates
class ICandidateList : public IDestructible
{
public:
	/// Return quantity of found candidates
	virtual int Size() const = 0;
	/// Get data about current candidate and move cursor to next
	/** @param [in]  pIt - pointer to cursor 
		@param [out] ppszText - pointer to candidate text
		@param [out] pWeight - weight of candidate 
		@return new cursor value, NULL if reached end of list */
	virtual const void* Next(const void *pIt, const char **ppszText, 
		Weight_t *pWeight = 0) const = 0;
	/// Return cursor that is reset to first candidate
	virtual const void* Start() const = 0;
	/// Return pointer to candidate text
	/** @param [in] nIndex - index of candidate
		@param [out] pWeight - weight of candidate */
	virtual const char *At(int nIndex, Weight_t *pWeight = 0) const = 0;
	///	Return language of current candidates
	/**	@return language of current candidates if spellchecking was successed, 
		else return language of the database, where last spellchecking occurred*/
	virtual const char *GetLanguage() const = 0;
};

/// Default flag value for skipping spell checking for some words
#define DEFAULT_SPELLING (SKIP_CORRECT_WORD | SKIP_WORD_NO_ALPHA)
/// Maximum allowed word length
#define MAXIMUM_SIZE_WORD (0xff - 0x3)

/// Check word spelling
class ISpellCheck : public IDestructible
{
public:
	/// Flags determining for which word speller should not find candidates
	enum ESkip_Err 
	{
		/// check all words
		SKIP_WORD_NO			=  0x0,	
		/// doesn't check word if it contains letter in upper case 
		SKIP_WORD_UPPER_CASE	=  0x1,	
		/// doesn't check word if it contains digit
		SKIP_WORD_NUM			=  0x2,	
		/// doesn't check word if speller dictionary contains it
		SKIP_CORRECT_WORD		=  0x4,	
		/// doesn't check word if it contains symbols other than letters and "-/.'` "
		SKIP_WORD_NO_ALPHA		=  0x8, 
		/// doesn't check word if all its letters is in upper case 
		SKIP_WORD_ALL_UPPER		= 0x10, 
	};
	/// flags determining which rules and dictionaries use when check spelling
	enum EDB
	{
		USE_ALL					= 0x0,	/// use all available resourses
		NOUSE_SOUNDEX			= 0x1,	/// don't use soundex submodule
		NOUSE_MISTYPER			= 0x2,	/// don't use mistyper submodule
		NOUSE_USER_DICTONARY	= 0x4,	/// don't use user dictionary   
		NOUSE_CONFIG_FILE	    = 0x8	/// don't use config file
	};
	/// Result of spell checking procedure
	enum ERes 
	{ 
		WORD_SKIP = 0,		/// spell checking is skipped 
		WORD_CORRECT = 1,	///	spelling is correct	
		WORD_INCORRECT = 2,	///	spelling is incorrect	
		WORD_BAD = 3		///  invalid word (too long or short)
	};
	/// mode defining how to adjust different spelling limitations
	enum ESpellTune
	{
		ETuneDefault = 0, /// set default values from LDB
		ETuneReduce = 1,  /// reduce bounds to given value
		ETuneEqual = 2,	  /// set all bounds to given value 
		ETuneRaise= 3	  /// raise bounds to given value
	};

	/// Check word spelling
	/** @param [in] pszWord  - text of checked word
		@param [out] ppiCandidates - pointer to interface to candidate list
		@param [in] pszLanguage - pointer to words language, 
			if pszLanguage = 0, check word using current language DB
		@return result of spell checking */
	virtual ERes CheckWord(const char *pszWord, ICandidateList **ppiCandidates = 0, const char *pszLanguage = 0) const = 0;
	/// Set maximum number of error that can be corrected in a word
	/** @param [in] eTune - mode determining how to adjust this bound for different word lengths
		@param [in] nBound - bound value */
	virtual void SetMaxError(ESpellTune eTune, int nBound = -1) = 0;
	/// Get maximum number of error that can be corrected in a word
	/** @param [out] pnBound - bound value 
		@return mode determining how to adjust this bound for different word lengths*/
	virtual ESpellTune GetMaxError(int* pnBound) const = 0;
	/// Set minimal weight for returned candidates
	/** @param [in] eTune - mode determining how to adjust this bound for different word lengths
		@param [in] nWeight - bound value */
	virtual void SetMinProbability(ESpellTune eTune, Weight_t nWeight) = 0;
	/// Get minimal weight for returned candidates
	/** @param [out] pWeight - bound value 
		@return mode determining how to adjust this bound for different word lengths*/
	virtual ESpellTune GetMinProbability(Weight_t* pWeight) const = 0;
	/// Set flags determining for which words skip spell checking
	/** @param [in] eSkip - flag value, see ESkip_Err enum*/
	virtual void SetSpellFilter(int eSkip) = 0;
	/// Return value of flags determining for which words skip spell checking
	virtual int GetSpellFilter() const = 0;
	/// Set new SpellChecker DB
	/** @param [in] pDB  - pointer to DB
		@param [in] fReset - flags determining which resources use in spell checking, see EDB enum */
	virtual void ResetDB(IDestructible *pDB, int fReset = USE_ALL) = 0;
	/// Enables or disables word processing using other languages if cheking with current language failed
	/**	@param [in] bFlag : true - endable other languages, false - disable other languages*/
	virtual void EnableAllLanguagesProcessing(bool bFlag) = 0;
}; 

/// Load SpellChecker LDB
/** @param [in] pszPath  - path to linguistic database directory
	@return pointer to LDB interface */
SEM_SDK_SPELLCHECKER_ IDestructible* LoadSpellCheckerDB(const char *pszPath);

/// Create spelling checker
/** @param  [in] pDB - pointer to LDB interface
	@param  [in] fReset  - flags determining which resources use in spell checking
	@return pointer to spelling checker interface */
SEM_SDK_SPELLCHECKER_ ISpellCheck* CreateSpellChecker(IDestructible* pDB, 
										int fReset = ISpellCheck::USE_ALL);

/// Get SpellChecker version.
/** @param [in/out] pszBuffer - pointer to the buffer to store the version string.
	@param [in] nBufferSize - size of the buffer.
	@param [in] pszPathLDB - path to LDB directory
	@return true in the case of success, or false otherwise. */
SEM_SDK_SPELLCHECKER_ bool GetSpellCheckerVersion(char* pszBuffer, 
							   int nBufferSize, const char *pszPathLDB);

/// Set path to license file
/** @param [in] pszFileName - to license file */
SEM_SDK_SPELLCHECKER_ void SetSpellCheckerLicensePath(const char* pszFileName);

/// Set license buffer
/** @param [in] pszBuffer - license buffer */
SEM_SDK_SPELLCHECKER_ void SetSpellCheckerLicenseBuffer(const char* pszBuffer);

} // namespace NsSemSDK

#endif //SEM_SDK_SPELLCHECKER__H_
