// File: SpellCheckerLib.h
// Description: this header provides common DLL interface for spellchecker
// Copyright: (C) 2005 EffectiveSoft Ltd. All Rights Reserved.
// Technical support: technical-support@effective-soft.com


#ifndef SPELLCHECKERLIB_INCLUDED
#define SPELLCHECKERLIB_INCLUDED

#ifdef SPELLCHECKERLIB_EXPORTS
#define SPELLCHECKERLIB_API __declspec(dllexport)
#else
#define SPELLCHECKERLIB_API __declspec(dllimport)
#endif


#ifdef __cplusplus
extern "C" {
#define STDCALL __stdcall
#else
#define STDCALL
#endif

enum ESPSkip_Err 
{
	/// check all words
	SKIP_WORD_NO			=  0x0,	
	/// doesn't check word if it contains letter in upper case 
	SKIP_WORD_UPPER_CASE	=  0x1,	
	/// doesn't check word if it contains digit
	SKIP_WORD_NUM			=  0x2,	
	/// doesn't check word if speller dictionary contains it
	SKIP_CORRECT_WORD		=  0x4,	
	/// doesn't check word if it contains symbols other than letters and "-/.'` "
	SKIP_WORD_NO_ALPHA		=  0x8, 
	/// doesn't check word if all its letters is in upper case 
	SKIP_WORD_ALL_UPPER		= 0x10, 
};
/// flags determining which rules and dictionaries use when check spelling
enum ESPDB
{
	USE_ALL					= 0x0,	/// use all available resourses
	NOUSE_SOUNDEX			= 0x1,	/// don't use soundex submodule
	NOUSE_MISTYPER			= 0x2,	/// don't use mistyper submodule
	NOUSE_USER_DICTONARY	= 0x4,	/// don't use user dictionary   
	NOUSE_CONFIG_FILE	    = 0x8	/// don't use config file
};
/// Result of spell checking procedure
enum ESPRes 
{ 
	WORD_SKIP = 0,			/// spell checking is skipped 
	WORD_CORRECT = 1,		///	spelling is correct	
	WORD_INCORRECT = 2,		///	spelling is incorrect	
	WORD_BAD = 3			/// invalid word (too long or short)
};

/// mode defining how to adjust different spelling limitations
enum ESPSpellTune
{
	ETuneError = -1,	/// error
	ETuneDefault = 0,	/// set default values from LDB
	ETuneReduce = 1,	/// reduce bounds to given value
	ETuneEqual = 2,		/// set all bounds to given value 
	ETuneRaise = 3		/// raise bounds to given value
};

/// Default flag value for skipping spell checking for some words
#define DEFAULT_SPELLING (SKIP_CORRECT_WORD | SKIP_WORD_NO_ALPHA)
/// Maximum allowed word length
#define MAXIMUM_SIZE_WORD (0xff - 0x3)

typedef unsigned int SPELL_DB_HANDLE;
typedef unsigned int SPELL_HANDLE;
typedef unsigned int SPELL_RESULT_HANDLE;

/// Get last error occured. You may call this function to get an error
/// string if any of functions below (except CheckWord) fail. 
/** @return Last error string or empty string if there was no error */
SPELLCHECKERLIB_API const char* STDCALL GetLastSpellCheckerError();

// Initialization

/// Load SpellChecker DB, after work call CloseSpellDBHandle to deallocate resources
/** @param [in] pszPath  - path to linguistic database (LDB) directory 
@return SpellChecker DB handle or NULL in the case of error*/
SPELLCHECKERLIB_API SPELL_DB_HANDLE STDCALL LoadSpellCheckerDB(const char *pszPath);

/// Closes SpellChecker DB handle
/** @param [in] hDB - SpellChecker DB handle **/
SPELLCHECKERLIB_API void STDCALL CloseSpellDBHandle(SPELL_DB_HANDLE hDB);

/// Create spelling checker, after work call CloseSpellHandle to deallocate resources
/** @param  [in] hDB - SpellChecker DB handle
@param  [in] fReset  - flags determining which resources use in spell checking, USE_ALL is the default value (see ESPDB enum)
@return SpellChecker handle*/
SPELLCHECKERLIB_API SPELL_HANDLE STDCALL CreateSpellChecker(SPELL_DB_HANDLE hDB,
													  int fReset);

/// Closes SpellChecker handle
/** @param [in] hSP - SpellChecker handle **/
SPELLCHECKERLIB_API void STDCALL CloseSpellHandle(SPELL_HANDLE hSP);

/// Get SpellChecker version.
/** @param [in/out] pszBuffer - pointer to the buffer to store the version string.
@param [in] nBufferSize - size of the buffer.
@param [in] pszPathLDB - path to LDB directory
@return true in the case of success, or false otherwise. **/
SPELLCHECKERLIB_API bool STDCALL GetSpellCheckerVersion(char* pszBuffer, 
												  int nBufferSize, const char *pszPathLDB);
/// Set path to license file
/** @param [in] pszFileName - path to license file **/
SPELLCHECKERLIB_API void STDCALL SetSpellCheckerLicensePath(const char* pszFileName);

/// Set license buffer
/** @param [in] pszBuffer - license buffer */
SPELLCHECKERLIB_API void STDCALL SetSpellCheckerLicenseBuffer(const char* pszBuffer);

// Spellchecking

/// Check word spelling; at the end of result processing call CloseResultHandle to deallocate resources.
/** @param [in] hSP - SpellChecker handle
@param [in] pszWord  - text of checked word
@param [in] pszLanguage - pointer to words language, if pszLanguage = 0, check word using current language DB
@return spellchecking result handle, you may use this handle to get result type and spelling candidates with weights **/
SPELLCHECKERLIB_API SPELL_RESULT_HANDLE STDCALL CheckWord(SPELL_HANDLE hSP, const char* pszWord, const char* pszLanguage = 0);

/// Enables or disables word processing in other languages if checking in current language failed
/**	@param [in] hSP - SpellChecker handle
@param [in] bFlag : true - endable other languages, false - disable other languages **/
SPELLCHECKERLIB_API void STDCALL EnableAllLanguagesProcessing(SPELL_HANDLE hSP, bool bFlag);

/// Get type of spellchecking result, see ESPRes enum
/** @param [in] hSR - spellchecking result handle
@return type of spellchecking result **/
SPELLCHECKERLIB_API ESPRes STDCALL GetResultType(SPELL_RESULT_HANDLE hSR);

/// Get count of candidate words that may correct spelling error
/** @param [in] hSR - spellchecking result handle 
@return count of candidate words **/
SPELLCHECKERLIB_API int STDCALL GetCandidateCount(SPELL_RESULT_HANDLE hSR);

/// Get candidate word at position nIndex
/** @param [in] hSR - spellchecking result handle 
@return text of candidate word **/
SPELLCHECKERLIB_API const char* STDCALL GetCandidateWord(SPELL_RESULT_HANDLE hSR, int nIndex);

/// Get weight of candidate word at position nIndex
/** @param [in] hSR - spellchecking result handle 
@return weight of candidate word **/
SPELLCHECKERLIB_API int STDCALL GetCandidateWeight(SPELL_RESULT_HANDLE hSR, int nIndex);

///	Return language of current candidates
/**	@return language of current candidates if spellchecking was successed, 
else return language of the database, where last spellchecking occurred*/
SPELLCHECKERLIB_API const char* STDCALL GetCandidateLanguage(SPELL_RESULT_HANDLE hSR);

/// Close spellchecking result handle 
/** @param [in] hSR - spellchecking result handle **/
SPELLCHECKERLIB_API void STDCALL CloseResultHandle(SPELL_RESULT_HANDLE hSR);

/// Settings

/// Set maximum number of errors that can be corrected in a word
/** @param [in] eTune - mode determining how to adjust this bound for different word lengths
@param [in] nBound - bound value */
SPELLCHECKERLIB_API void STDCALL SetMaxError(SPELL_HANDLE hSP, ESPSpellTune eTune, int nBound);

/// Get maximum number of errors that can be corrected in a word
/** @param [out] pnBound - bound value 
@return mode determining how to adjust this bound for different word lengths*/
SPELLCHECKERLIB_API ESPSpellTune STDCALL GetMaxError(SPELL_HANDLE hSP, int* pnBound);

/// Set minimal weight for returned candidates
/** @param [in] eTune - mode determining how to adjust this bound for different word lengths
@param [in] nWeight - bound value */
SPELLCHECKERLIB_API void STDCALL SetMinProbability(SPELL_HANDLE hSP, ESPSpellTune eTune, int nWeight);

/// Get minimal weight for returned candidates
/** @param [out] pnWeight - bound value 
@return mode determining how to adjust this bound for different word lengths*/
SPELLCHECKERLIB_API ESPSpellTune STDCALL GetMinProbability(SPELL_HANDLE hSP, int* pnWeight);

/// Set flags determining for which words skip spell checking
/** @param [in] eSkip - flag value, see ESPSkip_Err enum */
SPELLCHECKERLIB_API void STDCALL SetSpellFilter(SPELL_HANDLE hSP, int eSkip);

/// Return value of flags determining for which words skip spell checking
SPELLCHECKERLIB_API int STDCALL GetSpellFilter(SPELL_HANDLE hSP);

/// Set new SpellChecker DB
/** @param [in] pDB  - pointer to DB
@param [in] fReset - flags determining which resources use in spell checking, USE_ALL is the default value (see ESPDB enum) */
SPELLCHECKERLIB_API void STDCALL ResetDB(SPELL_HANDLE hSP, SPELL_DB_HANDLE hDB, int fReset);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //SPELLCHECKERLIB_INCLUDED
